package sample.persistence.web;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.wicket.util.convert.ConversionException;
import org.apache.wicket.util.convert.converters.SqlTimestampConverter;

public class PatternedSqlTimestampConverter extends SqlTimestampConverter {

	private final String pattern;
	
	public PatternedSqlTimestampConverter(String pattern) {
		this.pattern = pattern;
	}
	
	@Override
	public Object convertToObject(String value, Locale locale) {
		if (value == null) {
			return null;
		}
		if (locale == null) {
			locale = Locale.getDefault();
		}
		
		try {
			return new Timestamp(new SimpleDateFormat(pattern, locale).parse(value).getTime());
		}
		catch (ParseException e) {
			throw new ConversionException("Cannot parse '" + value + "' using format " + new SimpleDateFormat(pattern, locale))
					.setSourceValue(value).setTargetType(getTargetType()).setConverter(this)
					.setLocale(locale);
		}
	}

	@Override
	public String convertToString(Object value, Locale locale) {
		if (value == null) {
			return null;
		}
		if (locale == null) {
			locale = Locale.getDefault();
		}
		
		return new SimpleDateFormat(pattern, locale).format((Date)value);
	}
	
}
