package sample.persistence.web.order.page;

import java.util.List;

import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.convert.IConverter;

import sample.persistence.entity.ItemOrder;
import sample.persistence.service.OrderService;
import sample.persistence.web.PatternedSqlTimestampConverter;

public class OrderListPage extends WebPage {

	public transient OrderService orderService;
	
	public OrderListPage() {
		super();
		
		add(homePageLink("menu"));
		
		List<ItemOrder> orders = orderService.all();
		
		add(new ListView("orders", orders) {
			private int no = 1;

			@Override
			protected void populateItem(ListItem item) {
				final ItemOrder order = (ItemOrder)item.getModelObject();
				
				item.add(new Label("no", new Model(no++)));

				item.setModel(new CompoundPropertyModel(order));
				item.add(new Label("orderNo"));
				item.add(new Label("customerName"));
				item.add(new Label("orderDate") {
					@SuppressWarnings("unchecked")
					@Override
					public IConverter getConverter(Class type) {
						return new PatternedSqlTimestampConverter("yy/MM/dd");
					}
				});
				item.add(new Label("versionNo"));
				item.add(new Label("timestamp"));
				
				item.add(new Link("showDetail") {
					@Override
					public void onClick() {
						setResponsePage(new OrderPage(orderService.find(order.getOrderNo()), false));
					}
				});
			}
		});
		
		add(new FeedbackPanel("feedback"));
		
		info(" " + orders.size() + " ̒o^Ă܂.");
	}
	
}
