package sample.persistence.web.order.page;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormSubmitBehavior;
import org.apache.wicket.extensions.ajax.markup.html.autocomplete.AutoCompleteTextField;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.convert.IConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import sample.persistence.entity.Item;
import sample.persistence.entity.ItemOrder;
import sample.persistence.entity.OrderDetail;
import sample.persistence.service.ItemService;
import sample.persistence.service.OrderService;
import sample.persistence.web.ComponentEnableVisitor;
import sample.persistence.web.PatternedSqlTimestampConverter;

public class OrderPage extends WebPage {
	
	private static final Logger LOG = LoggerFactory.getLogger(OrderPage.class);

	private boolean editable;
	
	private ItemOrder order;
	
	public transient OrderService orderService;
	
	public transient ItemService itemService;
	
	public OrderPage() {
		this(new ItemOrder(), true);
	}
	
	public OrderPage(ItemOrder order, boolean editable) {
		super();
		this.editable = editable;
		this.order = order;
		
		add(homePageLink("menu"));
		
		add(new Order("order"));
		
		add(new FeedbackPanel("feedback"));
	}

	@Override
	protected void onBeforeRender() {
		super.onBeforeRender();

		/*
		 * MEMO ǂݎpŉʂJꂽꍇA͗{^𖳌.
		 * 
		 * ȂA#visitChildren()̓RXgN^ł͂ȂA#onBeforeRender()ŌĂяo.
		 * iListViewɏ]ListItemARXgN^̌ŐUIR|̂ŁA
		 * @RXgN^ŌĂяoĂAUIR|͖łȂj
		 */
		if (!editable) {
			disablePage();
		}
	}

	public class Order extends Form {
		
		public Order(String id) {
			super(id, new CompoundPropertyModel(order));
			
			add(new Label("orderNo"));
			add(new TextField("customerName"));
			add(new Label("orderDate") {
				@SuppressWarnings("unchecked")
				@Override
				public IConverter getConverter(Class type) {
					return new PatternedSqlTimestampConverter("yy/MM/dd");
				}
			});
			add(new Label("versionNo"));
			add(new Label("timestamp"));

			
			add(new ListView("orderDetails", order.getOrderDetails()) {
				@Override
				protected void populateItem(ListItem item) {
					final OrderDetail detail = (OrderDetail)item.getModelObject();
					
					item.setModel(new CompoundPropertyModel(detail));
					item.add(new Label("detailNo"));

					/*
					 * MEMO AjaxŁAʂ𓮓IɔfUIR|[lgϐ錾.
					 * 
					 * ʏAUIR|
					 * 
					 * @Label label = new Label("hoge", "ق");
					 * @add(label);
					 * 
					 * ̂悤ɁAϐ錾邱ƂȂA
					 * 
					 * @add(new Label("hoge", "ق"));
					 * 
					 * ̂悤ɃR[fBO.
					 * 
					 * AAjax𗘗pꍇ́AʂUIR|ɓIɔf邽߁A
					 * AjaxFormSubmitBehavior#onSubmit() ȂǂUIR|QƂłȂ΂ȂȂ̂ŁA
					 * Ajax̏ʂ󂯂UIR|́AO҂̂悤ɕϐ錾Ă.
					 */
					/*
					 * MEMO Ajaxʂ󂯎UIR|́A#setOutputMarkupId(true) ĂȂ΂ȂȂ.
					 */
					final Component itemName = new Label("itemName", new PropertyModel(detail, "item.itemName")).setOutputMarkupId(true);
					final Component price = new Label("price", new PropertyModel(detail, "item.price")).setOutputMarkupId(true);
					final Component amount = new Label("amount").setOutputMarkupId(true);

					/*
					 * MEMO Ajaxœ͌I[gRv[geLXgtB[h.
					 */
					item.add(new AutoCompleteTextField("itemCode", new PropertyModel(detail, "itemCode")) {
									@SuppressWarnings("unchecked")
									@Override
									protected Iterator getChoices(String input) {
										if (input == null || input.length() < 1) {
											return Collections.EMPTY_LIST.iterator();
										} else {
											return findCandidateItemCodes(input).iterator();
										}
									}
								}
								/*
								 * MEMO ͂ꂽlAAjaxŃnhO.
								 */
								.add(new AjaxFormSubmitBehavior((Form)getParent(), "onChange"){
									@Override
									protected void onError(AjaxRequestTarget target) {
										LOG.debug("AjaxFormSubmitBehavior#onError() - " + ToStringBuilder.reflectionToString(target));
									}
									@Override
									protected void onSubmit(AjaxRequestTarget target) {
										/*
										 * MEMO rwCrAŁAUIR|[lg̒l擾.
										 * @Behavior#getComponent()ŁArwCrAgݒ肳ĂUIR|[lg擾.
										 * AUIR|[lgAgetModel()AgetModelObject()AgetModelObjectAsString()ȂǂŒl擾.
										 */
										String inputedItemCode = getComponent().getModelObjectAsString();
										detail.setItem(itemService.find(inputedItemCode));
										
										/*
										 * MEMO Ajaxʂ́AfUIR|ݒ.
										 */
										target.addComponent(itemName);
										target.addComponent(price);
										target.addComponent(amount);
										target.addComponent(getForm().get("totalAmount"));
									}
								}));
						
					item.add(itemName);
					item.add(price);
					item.add(new TextField("quantity")
								.add(new AjaxFormSubmitBehavior((Form)getParent(), "onChange") {
									@Override
									protected void onError(AjaxRequestTarget target) {
										LOG.debug("AjaxFormSubmitBehavior#onError() - " + ToStringBuilder.reflectionToString(target));
									}
									@Override
									protected void onSubmit(AjaxRequestTarget target) {
										target.addComponent(amount);
										target.addComponent(getForm().get("totalAmount"));
									}
								}));
					item.add(amount);
				}
			});
			
			add(new Label("totalAmount").setOutputMarkupId(true));
			add(new Button("addRow") {
				@Override
				public void onSubmit() {
					order.addOrderDetail(new OrderDetail());
				}
			});
			
			
			add(new Button("register") {
				@Override
				public void onSubmit() {
					orderService.register(order);
					
					info("t܂Bԍ́A" + order.getOrderNo() + " łB");
					
					disablePage();
				}
			});
		}
	}
	
	List<String> findCandidateItemCodes(String itemCodeSnippet) {
		List<String> itemCodes = new ArrayList<String>();
		for (Item item : itemService.findLike(itemCodeSnippet)) {
			itemCodes.add(item.getItemCode());
		}
		return itemCodes;
	}
	
	void disablePage() {
		visitChildren(Form.class, new ComponentEnableVisitor(false, true));
	}
	
}
